#!/usr/bin/env python3
"""
AgentBeats package main entry point.

This module allows the package to be executed as a module with python -m agentbeats.
"""

from .cli import main

if __name__ == "__main__":
    main()