"""
Utility scripts for managing role/scenario matching data.

This package contains scripts for:
- Clearing role matching data from the database
- Populating role matches for existing agents
"""

__version__ = "1.0.0" 